@@ -1,3 +1,5 @@
+import collections
+
 from django import forms, template
 from django.forms.formsets import all_valid
 from django.forms.models import modelformset_factory
@@ -27,7 +29,10 @@ from django.utils.functional import curry
 from django.utils.text import capfirst, get_text_list
 from django.utils.translation import ugettext as _
 from django.utils.translation import ungettext
-from django.utils.encoding import force_unicode
+try:
+    from django.utils.encoding import force_text as force_unicode
+except ImportError:
+    from django.utils.encoding import force_unicode
 from django.forms.forms import pretty_name
 
 from mongoengine.fields import (DateTimeField, URLField, IntField, ListField, EmbeddedDocumentField, 
@@ -37,9 +42,11 @@ from mongodbforms.documentoptions import DocumentMetaWrapper
 from mongoadmin import mongohelpers
 from mongoadmin.util import RelationWrapper
 
-from mongodbforms.documents import (documentform_factory, DocumentForm, 
-                                  inlineformset_factory, BaseInlineDocumentFormSet)
+from mongodbforms.documents import (documentform_factory, DocumentForm, EmbeddedDocumentForm,
+                                  inlineformset_factory, BaseInlineDocumentFormSet, 
+                                  embeddedformset_factory, EmbeddedDocumentFormSet)
 from mongodbforms import MongoDefaultFormFieldGenerator, save_instance
+from mongodbforms.util import with_metaclass
 
 HORIZONTAL, VERTICAL = 1, 2
 # returns the <ul> class for a given radio_admin field
@@ -71,7 +78,7 @@ def formfield(field, form_class=None, **kwargs):
     """
     defaults = {'required': field.required, 'label': pretty_name(field.name)}
     if field.default is not None:
-        if callable(field.default):
+        if isinstance(field.default, collections.Callable):
             defaults['initial'] = field.default()
             defaults['show_hidden_initial'] = True 
         else:
@@ -84,7 +91,7 @@ def formfield(field, form_class=None, **kwargs):
         # Many of the subclass-specific formfield arguments (min_value,
         # max_value) don't apply for choice fields, so be sure to only pass
         # the values that TypedChoiceField will understand.
-        for k in kwargs.keys():
+        for k in list(kwargs.keys()):
             if k not in ('coerce', 'empty_value', 'choices', 'required',
                          'widget', 'label', 'initial', 'help_text',
                          'error_messages', 'show_hidden_initial'):
@@ -98,9 +105,8 @@ def formfield(field, form_class=None, **kwargs):
         return MongoDefaultFormFieldGenerator().generate(field, **defaults)
 
     
-class BaseDocumentAdmin(object):
+class BaseDocumentAdmin(with_metaclass(forms.MediaDefiningClass, object)):
     """Functionality common to both ModelAdmin and InlineAdmin."""
-    __metaclass__ = forms.MediaDefiningClass
 
     raw_id_fields = ()
     fields = None
@@ -237,7 +243,7 @@ class BaseDocumentAdmin(object):
         # ForeignKeyRawIdWidget, on the basis of ForeignKey.limit_choices_to,
         # are allowed to work.
         for l in model._meta.related_fkey_lookups:
-            for k, v in widgets.url_params_from_lookup_dict(l).items():
+            for k, v in list(widgets.url_params_from_lookup_dict(l).items()):
                 if k == lookup and v == value:
                     return True
 
@@ -317,10 +323,9 @@ class DocumentAdmin(BaseDocumentAdmin):
         
         self.model = document
         self.document = self.model
-        self.model._admin_opts = DocumentMetaWrapper(document)
-        self.model._meta = self.model._admin_opts 
+        self.document._meta = DocumentMetaWrapper(document)
         
-        self.opts = self.model._admin_opts
+        self.opts = self.document._meta
         
         self.admin_site = admin_site
         self.inline_instances = []
@@ -335,8 +340,9 @@ class DocumentAdmin(BaseDocumentAdmin):
         # Without this exclude is weirdly shared between all
         # instances derived from DocumentAdmin. 
         self.exclude = list(self.exclude)
-        self.get_inline_instances()
-                
+        if not isinstance(self, EmbeddedDocumentAdmin):
+            self.get_inline_instances()
+        
         # If someone patched their MAX_SHOW_ALL_ALLOWED in django 1.3, we 
         # get the value here and proceed as normal.
         try:
@@ -353,7 +359,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         return ret[0]
 
     def get_inline_instances(self):
-        for f in self.document._fields.itervalues():
+        for f in self.document._fields.values():
             if not (isinstance(f, ListField) and isinstance(getattr(f, 'field', None), EmbeddedDocumentField)) and not isinstance(f, EmbeddedDocumentField):
                 continue
             # Should only reach here if there is an embedded document...
@@ -488,7 +494,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         if self.declared_fieldsets:
             return self.declared_fieldsets
         form = self.get_form(request, obj)
-        fields = form.base_fields.keys() + list(self.get_readonly_fields(request, obj))
+        fields = list(form.base_fields.keys()) + list(self.get_readonly_fields(request, obj))
         return [(None, {'fields': fields})]
 
     def get_form(self, request, obj=None, **kwargs):
@@ -523,7 +529,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         """
         Returns the ChangeList class for use on the changelist page.
         """
-        from views import DocumentChangeList
+        from .views import DocumentChangeList
         return DocumentChangeList
 
     def get_object(self, request, object_id):
@@ -533,12 +539,13 @@ class DocumentAdmin(BaseDocumentAdmin):
         against the primary key field).
         """
         queryset = self.queryset(request)
-        model = queryset._document
-        model._admin_opts = DocumentMetaWrapper(model)
+        document = queryset._document
+        if not isinstance(document, DocumentMetaWrapper):
+            document._meta = DocumentMetaWrapper(document)
         try:
-            object_id = model._admin_opts.pk.to_python(object_id)
+            object_id = document._meta.pk.to_python(object_id)
             return queryset.get(pk=object_id)
-        except (model.DoesNotExist, ValidationError):
+        except (document.DoesNotExist, ValidationError):
             return None
 
     def get_changelist_form(self, request, **kwargs):
@@ -654,7 +661,7 @@ class DocumentAdmin(BaseDocumentAdmin):
             actions.extend([self.get_action(action) for action in class_actions])
 
         # get_action might have returned None, so filter any of those out.
-        actions = filter(None, actions)
+        actions = [_f for _f in actions if _f]
 
         # Convert the actions into a SortedDict keyed by name
         # and sorted by description.
@@ -671,7 +678,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         tuple (name, description).
         """
         choices = [] + default_choices
-        for func, name, description in self.get_actions(request).itervalues():
+        for func, name, description in self.get_actions(request).values():
             choice = (name, description % model_format_dict(self.opts))
             choices.append(choice)
         return choices
@@ -683,7 +690,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         (callable, name, description).
         """
         # If the action is a callable, just use it.
-        if callable(action):
+        if isinstance(action, collections.Callable):
             func = action
             action = action.__name__
 
@@ -775,7 +782,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         """
         Given a model instance save it to the database.
         """
-        save_instance(form, obj)
+        return save_instance(form=form, instance=obj)
         #obj.save()
 
     def delete_model(self, request, obj):
@@ -784,14 +791,14 @@ class DocumentAdmin(BaseDocumentAdmin):
         """
         obj.delete()
 
-    def save_formset(self, request, form, formset, change):
+    def save_formset(self, request, form, formset, change, commit=True):
         """
         Given an inline formset save it to the database.
         """
-        return formset.save()
+        return formset.save(commit=commit)
 
     def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
-        opts = self.model._admin_opts
+        opts = self.document._meta
         app_label = opts.app_label
         ordered_objects = opts.get_ordered_objects()
         context.update({
@@ -825,8 +832,8 @@ class DocumentAdmin(BaseDocumentAdmin):
         """
         Determines the HttpResponse for the add_view stage.
         """
-        opts = obj._admin_opts
-        pk_value = obj.pk.__str__()
+        opts = obj._meta
+        pk_value = str(obj.pk)
 
         msg = _('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
         # Here, we distinguish between different save types by checking for
@@ -860,7 +867,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         """
         Determines the HttpResponse for the change_view stage.
         """
-        opts = obj._admin_opts
+        opts = obj._meta
 
         verbose_name = opts.verbose_name
         # Handle proxy models automatically created by .only() or .defer()
@@ -868,7 +875,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         #    opts_ = opts.proxy_for_model._meta
         #    verbose_name = opts_.verbose_name
 
-        pk_value = obj.pk.__str__()
+        pk_value = str(obj.pk)
 
         msg = _('The %(name)s "%(obj)s" was changed successfully.') % {'name': force_unicode(verbose_name), 'obj': force_unicode(obj)}
         if "_continue" in request.POST:
@@ -966,8 +973,8 @@ class DocumentAdmin(BaseDocumentAdmin):
     @transaction.commit_on_success
     def add_view(self, request, form_url='', extra_context=None):
         "The 'add' admin view for this model."
-        model = self.model
-        opts = model._admin_opts
+        model = self.document
+        opts = model._meta
 
         if not self.has_add_permission(request):
             raise PermissionDenied
@@ -988,24 +995,36 @@ class DocumentAdmin(BaseDocumentAdmin):
                 prefixes[prefix] = prefixes.get(prefix, 0) + 1
                 if prefixes[prefix] != 1:
                     prefix = "%s-%s" % (prefix, prefixes[prefix])
-                formset = FormSet(data=request.POST, files=request.FILES,
-                                  instance=new_object,
-                                  save_as_new="_saveasnew" in request.POST,
-                                  prefix=prefix, queryset=inline.queryset(request))
-                formsets.append(formset)
+                    
+                formset_args = {
+                    "data": request.POST,
+                    "files": request.FILES,
+                    "prefix": prefix,
+                    "queryset": inline.queryset(request),
+                    "save_as_new": "_saveasnew" in request.POST,
+                }
+                if isinstance(inline, EmbeddedDocumentAdmin):
+                    formset_args["parent_document"] = new_object
+                else:
+                    formset_args["instance"] = new_object
+                formset = FormSet(**formset_args)
                 
-                if formset.is_valid() and form_validated:
-                    if isinstance(inline, EmbeddedDocumentAdmin):
-                        embedded_object_list = formset.save()
-                        if isinstance(inline.field, ListField):
-                            setattr(new_object, inline.rel_name, embedded_object_list)
-                        elif len(embedded_object_list) > 0:
-                            setattr(new_object, inline.rel_name, embedded_object_list[0])
-                        else:
-                            setattr(new_object, inline.rel_name, None)
+                if isinstance(inline, EmbeddedDocumentAdmin) and formset.is_valid() and form_validated:
+                    embedded_object_list = formset.save(commit=False)
+                    if isinstance(inline.field, ListField):
+                        setattr(new_object, inline.rel_name, embedded_object_list)
+                    elif len(embedded_object_list) > 0:
+                        setattr(new_object, inline.rel_name, embedded_object_list[0])
                     else:
-                        formset.save()
-                
+                        setattr(new_object, inline.rel_name, None)
+                            
+                    try:
+                        form._delete_before_save.remove(inline.rel_name)
+                    except (ValueError, AttributeError):
+                        pass
+                else:
+                    formsets.append(formset)
+                   
             if all_valid(formsets) and form_validated:
                 self.save_model(request, new_object, form, change=False)
                 for formset in formsets:
@@ -1017,7 +1036,7 @@ class DocumentAdmin(BaseDocumentAdmin):
         else:
             # Prepare the dict of initial data from the request.
             # We have to special-case M2Ms as a list of comma-separated PKs.
-            initial = dict(request.GET.items())
+            initial = dict(list(request.GET.items()))
             for k in initial:
                 try:
                     f = opts.get_field(k)
@@ -1034,8 +1053,18 @@ class DocumentAdmin(BaseDocumentAdmin):
                 prefixes[prefix] = prefixes.get(prefix, 0) + 1
                 if prefixes[prefix] != 1:
                     prefix = "%s-%s" % (prefix, prefixes[prefix])
-                formset = FormSet(instance=self.model(), prefix=prefix,
-                                  queryset=inline.queryset(request))
+                    
+                formset_args = {
+                    "prefix": prefix,
+                    "queryset": inline.queryset(request),
+                    "save_as_new": "_saveasnew" in request.POST,
+                }
+                if isinstance(inline, EmbeddedDocumentAdmin):
+                    formset_args["parent_document"] = inline.parent_document
+                else:
+                    formset_args["instance"] = self.model()
+                formset = FormSet(**formset_args)
+                
                 formsets.append(formset)
 
         adminForm = helpers.AdminForm(form, list(self.get_fieldsets(request)),
@@ -1070,8 +1099,8 @@ class DocumentAdmin(BaseDocumentAdmin):
     @transaction.commit_on_success
     def change_view(self, request, object_id, extra_context=None):
         "The 'change' admin view for this model."
-        model = self.model
-        opts = model._admin_opts
+        model = self.document
+        opts = model._meta
 
         obj = self.get_object(request, unquote(object_id))
 
@@ -1091,41 +1120,53 @@ class DocumentAdmin(BaseDocumentAdmin):
             form = DocumentForm(request.POST, request.FILES, instance=obj)
             if form.is_valid():
                 form_validated = True
-                new_object = self.save_form(request, form, change=True)
+                obj = self.save_form(request, form, change=True)
             else:
                 form_validated = False
-                new_object = obj
             prefixes = {}
-            for FormSet, inline in zip(self.get_formsets(request, new_object),
+            for FormSet, inline in zip(self.get_formsets(request, obj),
                                        self.inline_instances):
                 prefix = FormSet.get_default_prefix()
                 prefixes[prefix] = prefixes.get(prefix, 0) + 1
                 if prefixes[prefix] != 1:
                     prefix = "%s-%s" % (prefix, prefixes[prefix])
-                formset = FormSet(request.POST, request.FILES,
-                                  instance=new_object, prefix=prefix,
-                                  queryset=inline.queryset(request))
-
-                if formset.is_valid() and form_validated:
-                    if isinstance(inline, EmbeddedDocumentAdmin):
-                        embedded_object_list = formset.save()
-                        if isinstance(inline.field, ListField):
-                            setattr(new_object, inline.rel_name, embedded_object_list)
-                        elif len(embedded_object_list) > 0:
-                            setattr(new_object, inline.rel_name, embedded_object_list[0])
-                        else:
-                            setattr(new_object, inline.rel_name, None)
+                    
+                formset_args = {
+                    "data": request.POST,
+                    "files": request.FILES,
+                    "prefix": prefix,
+                    "queryset": inline.queryset(request),
+                }
+                if isinstance(inline, EmbeddedDocumentAdmin):
+                    formset_args["parent_document"] = obj
+                else:
+                    formset_args["instance"] = obj
+                formset = FormSet(**formset_args)
+                
+                if isinstance(inline, EmbeddedDocumentAdmin) and formset.is_valid() and form_validated:
+                    embedded_object_list = formset.save(commit=False)
+                    if isinstance(inline.field, ListField):
+                        setattr(obj, inline.rel_name, embedded_object_list)
+                    elif len(embedded_object_list) > 0:
+                        setattr(obj, inline.rel_name, embedded_object_list[0])
                     else:
-                        formset.save()
+                        setattr(obj, inline.rel_name, None)
+                            
+                    try:
+                        form._delete_before_save.remove(inline.rel_name)
+                    except (ValueError, AttributeError):
+                        pass
+                else:
+                    formsets.append(formset)
                         
             if all_valid(formsets) and form_validated:
-                self.save_model(request, new_object, form, change=True)
+                obj = self.save_model(request, obj, form, change=True)
                 for formset in formsets:
                     self.save_formset(request, form, formset, change=True)
 
                 change_message = self.construct_change_message(request, form, formsets)
-                self.log_change(request, new_object, change_message)
-                return self.response_change(request, new_object)
+                self.log_change(request, obj, change_message)
+                return self.response_change(request, obj)
 
         else:
             form = DocumentForm(instance=obj)
@@ -1137,8 +1178,16 @@ class DocumentAdmin(BaseDocumentAdmin):
                 prefixes[prefix] = prefixes.get(prefix, 0) + 1
                 if prefixes[prefix] != 1:
                     prefix = "%s-%s" % (prefix, prefixes[prefix])
-                formset = FormSet(instance=obj, prefix=prefix,
-                                  queryset=inline.queryset(request))
+                
+                formset_args = {
+                    "prefix": prefix,
+                    "queryset": inline.queryset(request),
+                }
+                if isinstance(inline, EmbeddedDocumentAdmin):
+                    formset_args["parent_document"] = obj
+                else:
+                    formset_args["instance"] = obj
+                formset = FormSet(**formset_args)
                 formsets.append(formset)
 
         adminForm = helpers.AdminForm(form, self.get_fieldsets(request, obj),
@@ -1204,7 +1253,7 @@ class DocumentAdmin(BaseDocumentAdmin):
             # and the 'invalid=1' parameter was already in the query string,
             # something is screwed up with the database, so display an error
             # page.
-            if ERROR_FLAG in request.GET.keys():
+            if ERROR_FLAG in list(request.GET.keys()):
                 return render_to_response('admin/invalid_setup.html', {'title': _('Database error')})
             return HttpResponseRedirect(request.path + '?' + ERROR_FLAG + '=1')
         
@@ -1325,7 +1374,7 @@ class DocumentAdmin(BaseDocumentAdmin):
     @transaction.commit_on_success
     def delete_view(self, request, object_id, extra_context=None):
         "The 'delete' admin view for this model."
-        opts = self.model._admin_opts
+        opts = self.document._meta
         app_label = opts.app_label
 
         obj = self.get_object(request, unquote(object_id))
@@ -1336,11 +1385,13 @@ class DocumentAdmin(BaseDocumentAdmin):
         if obj is None:
             raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
 
-        using = router.db_for_write(self.model)
-
         # Populate deleted_objects, a data structure of all related objects that
-        # will also be deleted.
-        print "FIXME: Need to delete nested objects."
+        # will also be deleted.        
+        #
+        #
+        # FIXME: Need to delete nested objects.
+        #
+        #
         #(deleted_objects, perms_needed, protected) = get_deleted_objects(
         #    [obj], opts, request.user, self.admin_site, using)
 
@@ -1397,8 +1448,8 @@ class DocumentAdmin(BaseDocumentAdmin):
 
         from mongoengine.base import ValidationError
         try:
-            obj = model.objects.get(id=object_id)
-        except ValidationError, e:
+            obj = model.objects.get(pk=object_id)
+        except ValidationError as e:
             raise Http404('No %s matches the given query.' % model._meta.object_name)
 
         context = {
@@ -1437,20 +1488,19 @@ class InlineDocumentAdmin(BaseDocumentAdmin):
     verbose_name_plural = None
     can_delete = True
 
-    def __init__(self, parent_document, admin_site):
+    def __init__(self, admin_site):
         self.admin_site = admin_site
-        self.parent_document = parent_document
-        if not hasattr(self.document, '_admin_opts'):
-            self.document._admin_opts = DocumentMetaWrapper(self.document)
-        self.opts = self.document._admin_opts
+        if not isinstance(self.document._meta, DocumentMetaWrapper):
+            self.document._meta = DocumentMetaWrapper(self.document)
+        self.opts = self.document._meta
          
         super(InlineDocumentAdmin, self).__init__()
         
         if self.verbose_name is None:
-            self.verbose_name = self.document._admin_opts.verbose_name
+            self.verbose_name = self.document._meta.verbose_name
         
         if self.verbose_name_plural is None:
-            self.verbose_name_plural = self.document._admin_opts.verbose_name_plural
+            self.verbose_name_plural = self.document._meta.verbose_name_plural
 
     def _media(self):
         from django.conf import settings
@@ -1495,10 +1545,13 @@ class InlineDocumentAdmin(BaseDocumentAdmin):
         if self.declared_fieldsets:
             return self.declared_fieldsets
         form = self.get_formset(request).form
-        fields = form.base_fields.keys() + list(self.get_readonly_fields(request, obj))
+        fields = list(form.base_fields.keys()) + list(self.get_readonly_fields(request, obj))
         return [(None, {'fields': fields})]
 
 class EmbeddedDocumentAdmin(InlineDocumentAdmin):
+    formset = EmbeddedDocumentFormSet
+    form = EmbeddedDocumentForm
+    
     def __init__(self, field, parent_document, admin_site):
         if hasattr(field, 'field'):
             self.document = field.field.document_type
@@ -1510,14 +1563,17 @@ class EmbeddedDocumentAdmin(InlineDocumentAdmin):
             self.doc_list = []
         self.rel_name = field.name
         
-        self.document._admin_opts = DocumentMetaWrapper(self.document)
+        if not isinstance(self.document._meta, DocumentMetaWrapper):
+            self.document._meta = DocumentMetaWrapper(self.document)
         if self.verbose_name is None:
-            self.verbose_name = "Field: %s (Document: %s)" % (capfirst(field.name), self.document._admin_opts.verbose_name)
+            self.verbose_name = "Field: %s (Document: %s)" % (capfirst(field.name), self.document._meta.verbose_name)
         
         if self.verbose_name_plural is None:
-            self.verbose_name_plural = "Field: %s (Document:  %s)" % (capfirst(field.name), self.document._admin_opts.verbose_name_plural)
+            self.verbose_name_plural = "Field: %s (Document:  %s)" % (capfirst(field.name), self.document._meta.verbose_name_plural)
         
-        super(EmbeddedDocumentAdmin, self).__init__(parent_document, admin_site)
+        self.parent_document = parent_document
+        
+        super(EmbeddedDocumentAdmin, self).__init__(admin_site)
         
     def queryset(self, request):
         if isinstance(self.field, ListField): # list field
@@ -1529,6 +1585,66 @@ class EmbeddedDocumentAdmin(InlineDocumentAdmin):
             else:
                 self.doc_list = [emb_doc]
         return self.doc_list
+    
+    def get_form(self, request, obj=None, **kwargs):
+        """
+        Returns a Form class for use in the admin add view. This is used by
+        add_view and change_view.
+        """
+        if self.declared_fieldsets:
+            fields = flatten_fieldsets(self.declared_fieldsets)
+        else:
+            fields = None
+        if self.exclude is None:
+            exclude = []
+        else:
+            exclude = list(self.exclude)
+        exclude.extend(kwargs.get("exclude", []))
+        exclude.extend(self.get_readonly_fields(request, obj))
+        # if exclude is an empty list we pass None to be consistant with the
+        # default on modelform_factory
+        exclude = exclude or None
+        defaults = {
+            "form": self.form,
+            "fields": fields,
+            "exclude": exclude,
+            "formfield_callback": curry(self.formfield_for_dbfield, request=request),
+        }
+        defaults.update(kwargs)
+        document = self.document()
+        form = documentform_factory(document, **defaults)
+        return form
+    
+    def get_formset(self, request, obj=None, **kwargs):
+        """Returns a BaseInlineFormSet class for use in admin add/change views."""
+        if self.declared_fieldsets:
+            fields = flatten_fieldsets(self.declared_fieldsets)
+        else:
+            fields = None
+        if self.exclude is None:
+            exclude = []
+        else:
+            exclude = list(self.exclude)
+        exclude.extend(kwargs.get("exclude", []))
+        exclude.extend(self.get_readonly_fields(request, obj))
+        # if exclude is an empty list we use None, since that's the actual default
+        exclude = exclude or None
+        if not hasattr(self.form, 'Meta') or not hasattr(self.form.Meta, 'embedded_field_name'):
+            self.form = self.get_form(request, obj)
+        defaults = {
+            "form": self.form,
+            "formset": self.formset,
+            "fields": fields,
+            "exclude": exclude,
+            "formfield_callback": curry(self.formfield_for_dbfield, request=request),
+            "extra": self.extra,
+            "max_num": self.max_num,
+            "can_delete": self.can_delete,
+        }
+        defaults.update(kwargs)
+        formset = embeddedformset_factory(self.document, self.parent_document, **defaults)
+        formset.form._meta.embedded_field = self.rel_name
+        return formset
 
 class StackedDocumentInline(InlineDocumentAdmin):
     template = 'admin/edit_inline/stacked.html'
