@@ -12,7 +12,8 @@ from mongodbforms.documentoptions import DocumentMetaWrapper
 from mongoengine.fields import ListField, EmbeddedDocumentField, ReferenceField
 from mongoengine.base import BaseDocument
 
-from options import BaseDocumentAdmin, EmbeddedDocumentAdmin
+from .options import BaseDocumentAdmin, EmbeddedDocumentAdmin
+import collections
 
 __all__ = ['validate']
 
@@ -39,7 +40,7 @@ def _validate(cls, model):
     if hasattr(cls, 'list_display'):
         check_isseq(cls, 'list_display', cls.list_display)
         for idx, field in enumerate(cls.list_display):
-            if not callable(field):
+            if not isinstance(field, collections.Callable):
                 if not hasattr(cls, field):
                     if not hasattr(model, field):
                         try:
@@ -333,7 +334,7 @@ def validate_base(cls, model):
     # radio_fields
     if hasattr(cls, 'radio_fields'):
         check_isdict(cls, 'radio_fields', cls.radio_fields)
-        for field, val in cls.radio_fields.items():
+        for field, val in list(cls.radio_fields.items()):
             f = get_field(cls, model, opts, 'radio_fields', field)
             if not (isinstance(f, models.ForeignKey) or f.choices):
                 raise ImproperlyConfigured("'%s.radio_fields['%s']' "
@@ -347,7 +348,7 @@ def validate_base(cls, model):
     # prepopulated_fields
     if hasattr(cls, 'prepopulated_fields'):
         check_isdict(cls, 'prepopulated_fields', cls.prepopulated_fields)
-        for field, val in cls.prepopulated_fields.items():
+        for field, val in list(cls.prepopulated_fields.items()):
             f = get_field(cls, model, opts, 'prepopulated_fields', field)
             if isinstance(f, (models.DateTimeField, models.ForeignKey,
                 models.ManyToManyField)):
@@ -404,7 +405,7 @@ def fetch_attr(cls, model, opts, label, field):
 def check_readonly_fields(cls, model, opts):
     check_isseq(cls, "readonly_fields", cls.readonly_fields)
     for idx, field in enumerate(cls.readonly_fields):
-        if not callable(field):
+        if not isinstance(field, collections.Callable):
             if not hasattr(cls, field):
                 if not hasattr(model, field):
                     try:
